@@ -1,5 +1,5 @@
 package ZMQ::Raw;
-$ZMQ::Raw::VERSION = '0.14';
+$ZMQ::Raw::VERSION = '0.15';
 use strict;
 use warnings;
 use Carp;
@@ -10,10 +10,12 @@ XSLoader::load ('ZMQ::Raw', $ZMQ::Raw::VERSION);
 use ZMQ::Raw::Context;
 use ZMQ::Raw::Curve;
 use ZMQ::Raw::Error;
+use ZMQ::Raw::Loop;
 use ZMQ::Raw::Message;
 use ZMQ::Raw::Poller;
 use ZMQ::Raw::Proxy;
 use ZMQ::Raw::Socket;
+use ZMQ::Raw::Timer;
 use ZMQ::Raw::Z85;
 
 sub AUTOLOAD
@@ -52,7 +54,7 @@ ZMQ::Raw - Perl bindings to the ZeroMQ library
 
 =head1 VERSION
 
-version 0.14
+version 0.15
 
 =head1 SYNOPSIS
 
